/* global describe, it */
'use strict';

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

require('./tests/frameworks');

var _bundle = require('./bundle');

var _bundle2 = _interopRequireDefault(_bundle);

var _decoratorsProvidersComponent = require('./decorators/providers/component');

var _decoratorsProvidersDirective = require('./decorators/providers/directive');

var _decoratorsInject = require('./decorators/inject');

var _module2 = require('./module');

var _classesProvider = require('./classes/provider');

describe('bundle a module', function () {
  var moduleAddStub = undefined;
  var MyApp = undefined;
  var Nested1 = undefined;
  var Nested2 = undefined;
  var fooProvider = undefined;

  beforeEach(function () {
    fooProvider = new _classesProvider.Provider('foo', { useValue: 'bar' });

    Nested2 = (function () {
      function Nested2() {
        _classCallCheck(this, _Nested2);
      }

      var _Nested2 = Nested2;
      Nested2 = (0, _decoratorsProvidersComponent.Component)({
        selector: 'nested2',
        providers: [['module-c'], fooProvider],
        events: ['event2']
      })(Nested2) || Nested2;
      return Nested2;
    })();

    Nested1 = (function () {
      function Nested1() {
        _classCallCheck(this, _Nested1);
      }

      var _Nested1 = Nested1;
      Nested1 = (0, _decoratorsProvidersComponent.Component)({
        selector: 'nested1',
        providers: ['module-b', [Nested2]],
        events: ['event1']
      })(Nested1) || Nested1;
      return Nested1;
    })();

    MyApp = (function () {
      function MyApp() {
        _classCallCheck(this, _MyApp);
      }

      var _MyApp = MyApp;
      MyApp = (0, _decoratorsProvidersComponent.Component)({
        selector: 'my-app',
        providers: ['module-a', Nested1]
      })(MyApp) || MyApp;
      return MyApp;
    })();

    moduleAddStub = sinon.stub(_module2.DecoratedModule.prototype, 'add').returnsThis();
  });

  afterEach(function () {
    moduleAddStub.restore();
  });

  it('with provided name', function () {
    var module = (0, _bundle2['default'])('foo', MyApp);
    module.name.should.be.equal('foo');
  });

  it('with required modules defined in providers from provider tree', function () {
    var module = (0, _bundle2['default'])('foo', MyApp);
    module._dependencies.should.be.eql(['module-a', 'module-b', 'module-c']);
  });

  it('with added annotated classes defined in providers from provider tree', function () {
    var module = (0, _bundle2['default'])('foo', MyApp)._module;
    var groupedByClass = moduleAddStub.args[0].reduce(function (memo, p) {
      var key = p.name || 'other';
      (memo[key] = memo[key] || []).push(p);
      return memo;
    }, {});

    groupedByClass.EventHandler.length.should.be.above(1);
    groupedByClass.MyApp.length.should.be.eql(1);
    groupedByClass.Nested1.length.should.be.eql(1);
    groupedByClass.Nested2.length.should.be.eql(1);
    groupedByClass.other.length.should.be.eql(1);
    groupedByClass.other[0].should.be.eql(fooProvider);
  });
});