/*global describe,it */
'use strict';

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

require('../tests/frameworks');

var _writers = require('../writers');

var _provider = require('./provider');

var _testsAngular = require('../tests/angular');

var _module2 = require('../module');

var _module3 = _interopRequireDefault(_module2);

var _bootstrap = require('../bootstrap');

var _bootstrap2 = _interopRequireDefault(_bootstrap);

var _decoratorsInject = require('../decorators/inject');

var _decoratorsProvidersInjectable = require('../decorators/providers/injectable');

var _decoratorsProvidersComponent = require('../decorators/providers/component');

var _tests = require('../tests');

var _classesOpaqueToken = require('../classes/opaque-token');

var _testsInternalUtils = require('../tests/internal-utils');

var SomeToken = function SomeToken() {
  _classCallCheck(this, SomeToken);
};

describe('Provider Class', function () {
  var someValue = undefined;
  var someConst = 'const';

  beforeEach(function () {
    _writers.providerWriter.set('type', 'whatever', SomeToken);
    _writers.providerWriter.set('name', 'someToken', SomeToken);
    someValue = 'val';
  });

  it('binds a decorated service token', function () {
    var b = new _provider.Provider(SomeToken, { useValue: true });
    b.token.should.equal('someToken');
  });

  it('binds a string-based service token', function () {
    var b = new _provider.Provider('$http', { useValue: true });
    b.token.should.equal('$http');
  });

  it('throws Invalid token if no token provided', function () {
    (function () {
      return new _provider.Provider().should['throw'];
    });
  });

  it('throws No usage provided if no use method is provided', function () {
    (function () {
      return new _provider.Provider('foo').should['throw'];
    });
  });

  it('binds to a value', function () {
    var b = new _provider.Provider(SomeToken, { useValue: someValue });
    b.useValue.should.equal('val');
  });

  it('binds to a constant', function () {
    var b = new _provider.Provider(SomeToken, { useConstant: someConst });
    b.useConstant.should.equal('const');
  });

  it('binds to a class', function () {
    var b = new _provider.Provider(SomeToken, { useClass: SomeToken });
    b.useClass.should.equal(SomeToken);
  });

  it('binds to a factory', function () {
    var b = new _provider.Provider(SomeToken, { useFactory: function useFactory() {
        return 'hi';
      } });
    b.useFactory().should.equal('hi');
  });

  describe('Angular Integration', function () {
    var root = undefined;

    describe('Provider\'s "use" Methods', function () {

      it('creates an angular value with useVal', function () {
        root = (0, _testsInternalUtils.quickRootTestComponent)({ providers: [new _provider.Provider('foo', { useValue: 'bar' })] });

        root.debugElement.getLocal('foo').should.eql('bar');
      });

      it('creates an angular constant with useConstant', function () {
        root = (0, _testsInternalUtils.quickRootTestComponent)({ providers: [new _provider.Provider('foo', { useConstant: 'bar' })] });

        root.debugElement.getLocal('foo').should.be.eql('bar');
      });

      it('creates an angular service with useClass', function () {
        var Bar = (function () {
          function Bar() {
            _classCallCheck(this, Bar);

            this.property = 'bar';
          }

          _createClass(Bar, [{
            key: 'method',
            value: function method() {
              return 'baz';
            }
          }]);

          return Bar;
        })();

        root = (0, _testsInternalUtils.quickRootTestComponent)({ providers: [new _provider.Provider('foo', { useClass: Bar })] });

        var foo = root.debugElement.getLocal('foo');
        foo.should.be.an.instanceOf(Bar);
        foo.property.should.be.eql('bar');
        foo.method().should.be.eql('baz');
      });

      it('creates an injected angular service with useClass', function () {
        var Bar = (function () {
          function Bar($q) {
            _classCallCheck(this, _Bar);

            this.$q = $q;
          }

          var _Bar = Bar;
          Bar = (0, _decoratorsInject.Inject)('$q')(Bar) || Bar;
          return Bar;
        })();

        root = (0, _testsInternalUtils.quickRootTestComponent)({ providers: [new _provider.Provider('foo', { useClass: Bar })] });

        var foo = root.debugElement.getLocal('foo');
        foo.should.be.an.instanceOf(Bar);
        foo.$q.should.be.eql(root.debugElement.getLocal('$q'));
        foo.$q.should.have.property('resolve');
      });

      it('creates an angular factory from a function with useFactory', function () {
        function getBar() {
          return 'bar';
        }

        root = (0, _testsInternalUtils.quickRootTestComponent)({ providers: [new _provider.Provider('foo', { useFactory: getBar })] });

        root.debugElement.getLocal('foo').should.eql('bar');
      });

      it('creates an angular factory with ng1 dependencies from a function with useFactory', function () {
        function getQ($q) {
          return $q;
        }

        root = (0, _testsInternalUtils.quickRootTestComponent)({ providers: [new _provider.Provider('foo', {
            useFactory: getQ,
            deps: ['$q']
          })] });

        root.debugElement.getLocal('foo').should.eql(root.debugElement.getLocal('$q'));
      });

      it('creates an angular factory with class dependencies from a function with useFactory', function () {
        var Foo = (function () {
          function Foo() {
            _classCallCheck(this, _Foo);

            this.bar = 'bar';
          }

          var _Foo = Foo;
          Foo = (0, _decoratorsProvidersInjectable.Injectable)()(Foo) || Foo;
          return Foo;
        })();

        function getBar(foo) {
          return foo.bar;
        }

        root = (0, _testsInternalUtils.quickRootTestComponent)({ providers: [new _provider.Provider('foo', {
            useFactory: getBar,
            deps: [Foo]
          })] });

        root.debugElement.getLocal('foo').should.eql('bar');
      });

      it('creates an angular factory with deep class dependencies from a function with useFactory', function () {
        var Baz = (function () {
          function Baz() {
            _classCallCheck(this, _Baz);

            this.quux = 'quux';
          }

          var _Baz = Baz;
          Baz = (0, _decoratorsProvidersInjectable.Injectable)()(Baz) || Baz;
          return Baz;
        })();

        var Bar = (function () {
          function Bar(baz) {
            _classCallCheck(this, _Bar2);

            this.baz = baz;
          }

          var _Bar2 = Bar;
          Bar = (0, _decoratorsInject.Inject)(Baz)(Bar) || Bar;
          Bar = (0, _decoratorsProvidersInjectable.Injectable)()(Bar) || Bar;
          return Bar;
        })();

        var Foo = (function () {
          function Foo(bar) {
            _classCallCheck(this, _Foo2);

            this.bar = bar;
          }

          var _Foo2 = Foo;
          Foo = (0, _decoratorsInject.Inject)(Bar)(Foo) || Foo;
          Foo = (0, _decoratorsProvidersInjectable.Injectable)()(Foo) || Foo;
          return Foo;
        })();

        function getQuux(foo) {
          return foo.bar.baz.quux;
        }

        root = (0, _testsInternalUtils.quickRootTestComponent)({ providers: [new _provider.Provider('getQuux', {
            useFactory: getQuux,
            deps: [Foo]
          })] });

        root.debugElement.getLocal('getQuux').should.eql('quux');
      });
    });

    describe('Provider Tokens', function () {

      it('supports string tokens', function () {
        root = (0, _testsInternalUtils.quickRootTestComponent)({ providers: [new _provider.Provider('foo', { useValue: 'bar' })] });

        root.debugElement.getLocal('foo').should.eql('bar');
      });

      it('supports OpaqueToken tokens', function () {
        var t = new _classesOpaqueToken.OpaqueToken('foo');
        root = (0, _testsInternalUtils.quickRootTestComponent)({ providers: [new _provider.Provider(t, { useValue: 'bar' })] });

        root.debugElement.getLocal(t).should.eql('bar');
      });

      it('supports class tokens', function () {
        var Foo = function Foo() {
          _classCallCheck(this, Foo);
        };

        root = (0, _testsInternalUtils.quickRootTestComponent)({ providers: [new _provider.Provider(Foo, { useValue: 'bar' })] });
        var name = _writers.providerWriter.get('name', Foo);
        root.debugElement.getLocal(name).should.be.eql('bar');
      });
    });
  });
});

describe('provide method', function () {
  it('aliases to new Provider() call', function () {
    var p = (0, _provider.provide)('bar', { useValue: 'val' });
    p.should.be.an.instanceOf(_provider.Provider);
    p.token.should.be.equal('bar');
    p.useValue.should.be.equal('val');
  });
});