/* global describe, it */
'use strict';

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

var _get = function get(_x, _x2, _x3) { var _again = true; _function: while (_again) { var object = _x, property = _x2, receiver = _x3; desc = parent = getter = undefined; _again = false; if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { _x = parent; _x2 = property; _x3 = receiver; _again = true; continue _function; } } else if ('value' in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } } };

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _inherits(subClass, superClass) { if (typeof superClass !== 'function' && superClass !== null) { throw new TypeError('Super expression must either be null or a function, not ' + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var _component = require('./component');

var _testsFrameworks = require('../../tests/frameworks');

var _testsAngular = require('../../tests/angular');

var _writers = require('../../writers');

var _classesProvider = require('../../classes/provider');

var _testsProviders = require('../../tests/providers');

var _utilEvents = require('../../util/events');

var _utilEvents2 = _interopRequireDefault(_utilEvents);

var _module2 = require('../../module');

var _module3 = _interopRequireDefault(_module2);

var _testsInternalUtils = require('../../tests/internal-utils');

var _utilEventEmitter = require('../../util/event-emitter');

describe('@Component', function () {
	var _this = this;

	describe('Annotation', function () {
		it('should throw an error if the selector is not provided', function () {
			var error = undefined;

			expect(function () {
				var MyClass = (function () {
					function MyClass() {
						_classCallCheck(this, _MyClass);
					}

					var _MyClass = MyClass;
					MyClass = (0, _component.Component)({})(MyClass) || MyClass;
					return MyClass;
				})();
			}).to['throw']('Component Decorator Error in "MyClass": Component selector must be provided');
		});

		it('should throw an error if the inputs, providers, directives, or outputs is not an array', function () {
			var providerParser = _module3['default'].getParser('component');
			var module = _testsAngular.ng.module();
			module.name = "myModule";

			expect(function () {
				var MyClass = (function () {
					function MyClass() {
						_classCallCheck(this, _MyClass2);
					}

					var _MyClass2 = MyClass;
					MyClass = (0, _component.Component)({ selector: 'my-component', inputs: {} })(MyClass) || MyClass;
					return MyClass;
				})();
			}).to['throw']('Component Decorator Error in "MyClass": Component inputs must be an array');

			expect(function () {
				var MyClass = (function () {
					function MyClass() {
						_classCallCheck(this, _MyClass3);
					}

					var _MyClass3 = MyClass;
					MyClass = (0, _component.Component)({ selector: 'my-component', providers: {} })(MyClass) || MyClass;
					return MyClass;
				})();
			}).to['throw']('Component Decorator Error in "MyClass": Component providers must be an array');

			expect(function () {
				var MyClass = (function () {
					function MyClass() {
						_classCallCheck(this, _MyClass4);
					}

					var _MyClass4 = MyClass;
					MyClass = (0, _component.Component)({ selector: 'my-component', directives: {} })(MyClass) || MyClass;
					return MyClass;
				})();
			}).to['throw']('Component Decorator Error in "MyClass": Component directives must be an array');

			expect(function () {
				var MyClass = (function () {
					function MyClass() {
						_classCallCheck(this, _MyClass5);
					}

					var _MyClass5 = MyClass;
					MyClass = (0, _component.Component)({ selector: 'my-component', outputs: {} })(MyClass) || MyClass;
					return MyClass;
				})();
			}).to['throw']('Component Decorator Error in "MyClass": Component outputs must be an array');
		});

		it('should decorate a class with correct provider metadata', function () {
			var MyComponentCtrl = (function () {
				function MyComponentCtrl() {
					_classCallCheck(this, _MyComponentCtrl);
				}

				var _MyComponentCtrl = MyComponentCtrl;
				MyComponentCtrl = (0, _component.Component)({ selector: 'my-component' })(MyComponentCtrl) || MyComponentCtrl;
				return MyComponentCtrl;
			})();

			_writers.providerWriter.get('type', MyComponentCtrl).should.eql('component');
			_writers.providerWriter.get('name', MyComponentCtrl).should.eql('myComponent');
		});

		it('should decorate a class with correct app metadata', function () {
			var p = new _classesProvider.Provider('foo', { useValue: 'bar' });

			var MyComponentCtrl = (function () {
				function MyComponentCtrl() {
					_classCallCheck(this, _MyComponentCtrl2);
				}

				var _MyComponentCtrl2 = MyComponentCtrl;
				MyComponentCtrl = (0, _component.Component)({
					selector: 'my-component',
					providers: ['some-module', p]
				})(MyComponentCtrl) || MyComponentCtrl;
				return MyComponentCtrl;
			})();

			_writers.appWriter.get('selector', MyComponentCtrl).should.eql('my-component');
			_writers.appWriter.get('modules', MyComponentCtrl).should.eql(['some-module']);
			_writers.appWriter.get('providers', MyComponentCtrl).should.eql([p]);
		});

		it('should decorate a class with inherited app metadata', function () {
			var p1 = new _classesProvider.Provider('foo', { useValue: 'bar' });
			var p2 = new _classesProvider.Provider('baz', { useValue: 'quux' });

			var ParentCtrl = (function () {
				function ParentCtrl() {
					_classCallCheck(this, _ParentCtrl);
				}

				var _ParentCtrl = ParentCtrl;
				ParentCtrl = (0, _component.Component)({
					selector: 'my-parent',
					providers: ['a-module', p1]
				})(ParentCtrl) || ParentCtrl;
				return ParentCtrl;
			})();

			var ChildCtrl = (function (_ParentCtrl2) {
				_inherits(ChildCtrl, _ParentCtrl2);

				function ChildCtrl() {
					_classCallCheck(this, _ChildCtrl);

					_get(Object.getPrototypeOf(_ChildCtrl.prototype), 'constructor', this).apply(this, arguments);
				}

				var _ChildCtrl = ChildCtrl;
				ChildCtrl = (0, _component.Component)({
					selector: 'my-child',
					providers: ['b-module', p2]
				})(ChildCtrl) || ChildCtrl;
				return ChildCtrl;
			})(ParentCtrl);

			_writers.appWriter.get('modules', ChildCtrl).should.eql(['b-module', 'a-module']);
			_writers.appWriter.get('providers', ChildCtrl).should.eql([p2, p1]);
		});

		it('should set sensible defaults using component metadata', function () {
			var MyComponentCtrl = (function () {
				function MyComponentCtrl() {
					_classCallCheck(this, _MyComponentCtrl3);
				}

				var _MyComponentCtrl3 = MyComponentCtrl;
				MyComponentCtrl = (0, _component.Component)({ selector: 'my-component' })(MyComponentCtrl) || MyComponentCtrl;
				return MyComponentCtrl;
			})();

			_writers.componentWriter.get('restrict', MyComponentCtrl).should.eql('E');
			_writers.componentWriter.get('scope', MyComponentCtrl).should.eql({});
			_writers.componentWriter.get('bindToController', MyComponentCtrl).should.be.ok;
		});

		it('should set component inputs metadata, respecting inheritance', function () {
			var ParentCtrl = (function () {
				function ParentCtrl() {
					_classCallCheck(this, _ParentCtrl3);
				}

				var _ParentCtrl3 = ParentCtrl;
				ParentCtrl = (0, _component.Component)({
					selector: 'parent',
					inputs: ['first', 'second']
				})(ParentCtrl) || ParentCtrl;
				return ParentCtrl;
			})();

			var ChildCtrl = (function (_ParentCtrl4) {
				_inherits(ChildCtrl, _ParentCtrl4);

				function ChildCtrl() {
					_classCallCheck(this, _ChildCtrl2);

					_get(Object.getPrototypeOf(_ChildCtrl2.prototype), 'constructor', this).apply(this, arguments);
				}

				var _ChildCtrl2 = ChildCtrl;
				ChildCtrl = (0, _component.Component)({ selector: 'child', inputs: ['third'] })(ChildCtrl) || ChildCtrl;
				return ChildCtrl;
			})(ParentCtrl);

			_writers.componentWriter.get('inputMap', ChildCtrl).should.eql({
				first: 'first',
				second: 'second',
				third: 'third'
			});
		});

		it('should set component outputs metadata', function () {
			var MyComponentCtrl = (function () {
				function MyComponentCtrl() {
					_classCallCheck(this, _MyComponentCtrl4);
				}

				var _MyComponentCtrl4 = MyComponentCtrl;
				MyComponentCtrl = (0, _component.Component)({ selector: 'child', outputs: ['first'] })(MyComponentCtrl) || MyComponentCtrl;
				return MyComponentCtrl;
			})();

			_writers.componentWriter.get('outputMap', MyComponentCtrl).should.eql({
				first: 'first'
			});

			var resolvedEvents = _utilEvents2['default'].resolve();
			var lastEvent = resolvedEvents[resolvedEvents.length - 1];
			_writers.providerWriter.get('name', lastEvent).should.eql('(first)');
		});

		it('should set component controllerAs metadata to camelCased selector', function () {
			var MyComponentCtrl1 = (function () {
				function MyComponentCtrl1() {
					_classCallCheck(this, _MyComponentCtrl1);
				}

				var _MyComponentCtrl1 = MyComponentCtrl1;
				MyComponentCtrl1 = (0, _component.Component)({ selector: 'foo' })(MyComponentCtrl1) || MyComponentCtrl1;
				return MyComponentCtrl1;
			})();

			var MyComponentCtrl2 = (function () {
				function MyComponentCtrl2() {
					_classCallCheck(this, _MyComponentCtrl22);
				}

				var _MyComponentCtrl22 = MyComponentCtrl2;
				MyComponentCtrl2 = (0, _component.Component)({ selector: 'foo-bar' })(MyComponentCtrl2) || MyComponentCtrl2;
				return MyComponentCtrl2;
			})();

			_writers.componentWriter.get('controllerAs', MyComponentCtrl1).should.eql('foo');
			_writers.componentWriter.get('controllerAs', MyComponentCtrl2).should.eql('fooBar');
		});

		it('should set component controllerAs metadata to custom', function () {
			var MyComponentCtrl1 = (function () {
				function MyComponentCtrl1() {
					_classCallCheck(this, _MyComponentCtrl12);
				}

				var _MyComponentCtrl12 = MyComponentCtrl1;
				MyComponentCtrl1 = (0, _component.Component)({ selector: 'foo', controllerAs: 'vm' })(MyComponentCtrl1) || MyComponentCtrl1;
				return MyComponentCtrl1;
			})();

			_writers.componentWriter.get('controllerAs', MyComponentCtrl1).should.eql('vm');
		});

		it('should set component link metadata if provided', function () {
			var MyComponentCtrl1 = (function () {
				function MyComponentCtrl1() {
					_classCallCheck(this, _MyComponentCtrl13);
				}

				var _MyComponentCtrl13 = MyComponentCtrl1;
				MyComponentCtrl1 = (0, _component.Component)({ selector: 'foo' })(MyComponentCtrl1) || MyComponentCtrl1;
				return MyComponentCtrl1;
			})();

			var MyComponentCtrl2 = (function () {
				function MyComponentCtrl2() {
					_classCallCheck(this, _MyComponentCtrl23);
				}

				_createClass(MyComponentCtrl2, null, [{
					key: 'link',
					value: function link() {}
				}]);

				var _MyComponentCtrl23 = MyComponentCtrl2;
				MyComponentCtrl2 = (0, _component.Component)({ selector: 'foo' })(MyComponentCtrl2) || MyComponentCtrl2;
				return MyComponentCtrl2;
			})();

			should.not.exist(_writers.componentWriter.get('link', MyComponentCtrl1));
			_writers.componentWriter.get('link', MyComponentCtrl2).should.equal(MyComponentCtrl2.link);
		});

		it('should set component compile metadata if provided', function () {
			var MyComponentCtrl1 = (function () {
				function MyComponentCtrl1() {
					_classCallCheck(this, _MyComponentCtrl14);
				}

				var _MyComponentCtrl14 = MyComponentCtrl1;
				MyComponentCtrl1 = (0, _component.Component)({ selector: 'foo' })(MyComponentCtrl1) || MyComponentCtrl1;
				return MyComponentCtrl1;
			})();

			var MyComponentCtrl2 = (function () {
				function MyComponentCtrl2() {
					_classCallCheck(this, _MyComponentCtrl24);
				}

				_createClass(MyComponentCtrl2, null, [{
					key: 'compile',
					value: function compile() {}
				}]);

				var _MyComponentCtrl24 = MyComponentCtrl2;
				MyComponentCtrl2 = (0, _component.Component)({ selector: 'foo' })(MyComponentCtrl2) || MyComponentCtrl2;
				return MyComponentCtrl2;
			})();

			should.not.exist(_writers.componentWriter.get('compile', MyComponentCtrl1));
			_writers.componentWriter.get('compile', MyComponentCtrl2).should.equal(MyComponentCtrl2.compile);
		});
	});

	describe('Angular Integration', function () {

		var angular = undefined;

		beforeEach(function () {
			angular = _testsAngular.ng.useReal();
		});

		beforeEach((0, _testsProviders.providers)(function (provide) {
			var $templateCache = {
				get: _testsFrameworks.sinon.stub(),
				put: _testsFrameworks.sinon.stub()
			};

			$templateCache.get.withArgs('foo.html').returns('template content');

			return [provide('$templateCache', { useValue: $templateCache })];
		}));

		it('should throw an error if the selector is not an element', function () {
			expect(function () {
				var MyClass = (function () {
					function MyClass() {
						_classCallCheck(this, _MyClass6);
					}

					var _MyClass6 = MyClass;
					MyClass = (0, _component.Component)({ selector: 'my-element', template: 'x' })(MyClass) || MyClass;
					return MyClass;
				})();

				(0, _testsInternalUtils.quickRootTestComponent)({ directives: [MyClass] });
			}).not.to['throw'](Error);

			expect(function () {
				var MyClass = (function () {
					function MyClass() {
						_classCallCheck(this, _MyClass7);
					}

					var _MyClass7 = MyClass;
					MyClass = (0, _component.Component)({ selector: '[my-attr]', template: 'x' })(MyClass) || MyClass;
					return MyClass;
				})();

				(0, _testsInternalUtils.quickRootTestComponent)({ directives: [MyClass] });
			}).to['throw']('Processing "MyClass" in "test.module": ' + '@Component selectors can only be elements. Perhaps you meant to use @Directive?');

			expect(function () {
				var MyClass = (function () {
					function MyClass() {
						_classCallCheck(this, _MyClass8);
					}

					var _MyClass8 = MyClass;
					MyClass = (0, _component.Component)({ selector: '.my-class', template: 'x' })(MyClass) || MyClass;
					return MyClass;
				})();

				(0, _testsInternalUtils.quickRootTestComponent)({ directives: [MyClass] });
			}).to['throw']('Processing "MyClass" in "test.module": ' + '@Component selectors can only be elements. Perhaps you meant to use @Directive?');
		});

		it('should throw an error if no template is provided', function () {
			expect(function () {
				var MyClass = (function () {
					function MyClass() {
						_classCallCheck(this, _MyClass9);
					}

					var _MyClass9 = MyClass;
					MyClass = (0, _component.Component)({ selector: 'my-component', template: true })(MyClass) || MyClass;
					return MyClass;
				})();

				(0, _testsInternalUtils.quickRootTestComponent)({ directives: [MyClass] });
			}).not.to['throw'];

			expect(function () {
				var MyClass = (function () {
					function MyClass() {
						_classCallCheck(this, _MyClass10);
					}

					var _MyClass10 = MyClass;
					MyClass = (0, _component.Component)({ selector: 'my-component', templateUrl: true })(MyClass) || MyClass;
					return MyClass;
				})();

				(0, _testsInternalUtils.quickRootTestComponent)({ directives: [MyClass] });
			}).not.to['throw'];

			expect(function () {
				var MyClass = (function () {
					function MyClass() {
						_classCallCheck(this, _MyClass11);
					}

					var _MyClass11 = MyClass;
					MyClass = View({ template: true })(MyClass) || MyClass;
					MyClass = (0, _component.Component)({ selector: 'my-component' })(MyClass) || MyClass;
					return MyClass;
				})();

				(0, _testsInternalUtils.quickRootTestComponent)({ directives: [MyClass] });
			}).not.to['throw'];

			expect(function () {
				var MyClass = (function () {
					function MyClass() {
						_classCallCheck(this, _MyClass12);
					}

					var _MyClass12 = MyClass;
					MyClass = View({ templateUrl: true })(MyClass) || MyClass;
					MyClass = (0, _component.Component)({ selector: 'my-component' })(MyClass) || MyClass;
					return MyClass;
				})();

				(0, _testsInternalUtils.quickRootTestComponent)({ directives: [MyClass] });
			}).not.to['throw'];

			expect(function () {
				var MyClass = (function () {
					function MyClass() {
						_classCallCheck(this, _MyClass13);
					}

					var _MyClass13 = MyClass;
					MyClass = (0, _component.Component)({ selector: 'my-component' })(MyClass) || MyClass;
					return MyClass;
				})();

				(0, _testsInternalUtils.quickRootTestComponent)({ directives: [MyClass] });
			}).to['throw']('Processing "MyClass" in "test.module": ' + 'Components must have a template or templateUrl via the @Component or @View decorators');
		});

		it('creates a directive with defaults', function () {
			var MyClass = (function () {
				function MyClass() {
					_classCallCheck(this, _MyClass14);
				}

				var _MyClass14 = MyClass;
				MyClass = (0, _component.Component)({ selector: 'foo', template: 'x' })(MyClass) || MyClass;
				return MyClass;
			})();

			var root = (0, _testsInternalUtils.quickRootTestComponent)({
				directives: [MyClass],
				template: '<foo></foo>'
			});

			var directive = root.debugElement.getLocal('fooDirective')[0];
			directive.restrict.should.eql('E');
			directive.scope.should.eql({});
		});

		it('creates a directive with name from selector', function () {
			var MyClass = (function () {
				function MyClass() {
					_classCallCheck(this, _MyClass15);
				}

				var _MyClass15 = MyClass;
				MyClass = (0, _component.Component)({ selector: 'foo', template: 'x' })(MyClass) || MyClass;
				return MyClass;
			})();

			var root = (0, _testsInternalUtils.quickRootTestComponent)({
				directives: [MyClass],
				template: '<foo></foo>'
			});

			var directive = root.debugElement.getLocal('fooDirective')[0];
			directive.name.should.eql('foo');
		});

		it('creates a directive with controllerAs created from selector', function () {
			var MyClass = (function () {
				function MyClass() {
					_classCallCheck(this, _MyClass16);
				}

				var _MyClass16 = MyClass;
				MyClass = (0, _component.Component)({ selector: 'foo', template: 'x' })(MyClass) || MyClass;
				return MyClass;
			})();

			var root = (0, _testsInternalUtils.quickRootTestComponent)({
				directives: [MyClass],
				template: '<foo></foo>'
			});

			var directive = root.debugElement.getLocal('fooDirective')[0];
			directive.controllerAs.should.eql('foo');
		});

		it('creates a directive with controllerAs that is provided', function () {
			var MyClass = (function () {
				function MyClass() {
					_classCallCheck(this, _MyClass17);
				}

				var _MyClass17 = MyClass;
				MyClass = (0, _component.Component)({ selector: 'foo', controllerAs: 'vm', template: 'x' })(MyClass) || MyClass;
				return MyClass;
			})();

			var root = (0, _testsInternalUtils.quickRootTestComponent)({
				directives: [MyClass],
				template: '<foo></foo>'
			});

			var directive = root.debugElement.getLocal('fooDirective')[0];
			directive.controllerAs.should.eql('vm');
		});

		it('creates a directive with multi-word selector that is provided', function () {
			var MyClass = (function () {
				function MyClass() {
					_classCallCheck(this, _MyClass18);
				}

				var _MyClass18 = MyClass;
				MyClass = (0, _component.Component)({ selector: 'foo-bar', template: 'x' })(MyClass) || MyClass;
				return MyClass;
			})();

			var root = (0, _testsInternalUtils.quickRootTestComponent)({
				directives: [MyClass],
				template: '<foo></foo>'
			});

			var directive = root.debugElement.getLocal('fooBarDirective')[0];
			directive.controllerAs.should.eql('fooBar');
		});

		it('creates a directive whose controller is an instance of the decorated class', function () {
			var MyClass = (function () {
				function MyClass() {
					_classCallCheck(this, _MyClass19);
				}

				var _MyClass19 = MyClass;
				MyClass = (0, _component.Component)({ selector: 'foo', template: true })(MyClass) || MyClass;
				return MyClass;
			})();

			var root = (0, _testsInternalUtils.quickRootTestComponent)({
				directives: [MyClass],
				template: '<foo></foo>'
			});

			var component = root.debugElement.componentViewChildren[0].componentInstance;
			component.should.be.instanceOf(MyClass);
		});

		it('creates a directive with the provided template', function () {
			var MyClass = (function () {
				function MyClass() {
					_classCallCheck(this, _MyClass20);
				}

				var _MyClass20 = MyClass;
				MyClass = (0, _component.Component)({ selector: 'foo', template: "foo" })(MyClass) || MyClass;
				return MyClass;
			})();

			var root = (0, _testsInternalUtils.quickRootTestComponent)({
				directives: [MyClass],
				template: '<foo></foo>'
			});

			var componentEl = root.debugElement.componentViewChildren[0];
			componentEl.html().should.be.eql('foo');
		});

		it('creates a directive with the provided templateUrl', function () {
			var MyClass = (function () {
				function MyClass() {
					_classCallCheck(this, _MyClass21);
				}

				var _MyClass21 = MyClass;
				MyClass = (0, _component.Component)({ selector: 'foo', templateUrl: "foo.html" })(MyClass) || MyClass;
				return MyClass;
			})();

			var root = (0, _testsInternalUtils.quickRootTestComponent)({
				directives: [MyClass],
				template: '<foo></foo>'
			});

			var componentEl = root.debugElement.componentViewChildren[0];
			componentEl.html().should.be.eql('template content');
		});

		describe('inputs', function () {

			it('adds each input as an allowed attribute on the element', function () {
				var MyClass = (function () {
					function MyClass() {
						_classCallCheck(this, _MyClass22);
					}

					var _MyClass22 = MyClass;
					MyClass = (0, _component.Component)({ selector: 'foo', template: '{{foo.bar}} {{foo.baz}}', inputs: ['bar', 'baz'] })(MyClass) || MyClass;
					return MyClass;
				})();

				var root = (0, _testsInternalUtils.quickRootTestComponent)({
					directives: [MyClass],
					template: '<foo bar="1" baz="2"></foo>'
				});

				root.debugElement.text().should.eql('1 2');
			});

			it.skip('disallows setting instance properties not marked as an input', function () {
				var MyClass = (function () {
					function MyClass() {
						_classCallCheck(this, _MyClass23);

						//uncomment this and the test passes, WTF!
						//something to do with saving a reference to this probably
						//console.log('constructor:', this);
						this.bar = "false";
						this.baz = "false";
					}

					var _MyClass23 = MyClass;
					MyClass = (0, _component.Component)({ selector: 'foo', template: '{{foo.bar}} {{foo.baz}}', inputs: ['baz'] })(MyClass) || MyClass;
					return MyClass;
				})();

				var root = (0, _testsInternalUtils.quickRootTestComponent)({
					directives: [MyClass],
					template: '<foo bar="true" baz="true"></foo>'
				});

				root.debugElement.text().should.eql('false true');
			});

			it('one way binds to an expression to inputs with the [input] syntax');
			it('allows changing local property on one way [input] until new input is received');
			it('two way binds an expression to inputs with the [(input)] syntax');
			it('allows using the desugared syntax [input]="prop" (input-change)="prop=$event" for two way binding');

			describe('binding to scope or bindToController based on angular version', function () {
				var quickBuildBindingTest = function quickBuildBindingTest() {
					var MyClass = (function () {
						function MyClass() {
							_classCallCheck(this, _MyClass24);
						}

						var _MyClass24 = MyClass;
						MyClass = (0, _component.Component)({ selector: 'foo', template: '{{foo.bar}}', inputs: ['bar'] })(MyClass) || MyClass;
						return MyClass;
					})();

					var root = (0, _testsInternalUtils.quickRootTestComponent)({
						directives: [MyClass],
						template: '<foo bar="baz"></foo>'
					});

					root.debugElement.text().should.eql('baz');
					return root.debugElement.getLocal('fooDirective')[0];
				};

				it('creates a directive putting bindings on scope object in angular 1.3', function () {
					angular.version.minor = 3;

					var directive = quickBuildBindingTest();
					directive.scope.should.not.be.empty;
					directive.bindToController.should.be['true'];
				});

				it('creates a directive putting bindings on bindToController object in angular 1.4', function () {
					angular.version.minor = 4;

					var directive = quickBuildBindingTest();
					directive.scope.should.be.empty;
					directive.bindToController.should.not.be.empty;
				});

				it('creates a directive putting bindings on bindToController object in future angular versions', function () {
					angular.version.minor = 9;

					var directive = quickBuildBindingTest();
					directive.scope.should.be.empty;
					directive.bindToController.should.not.be.empty;
				});
			});
		});

		describe('outputs', function () {
			beforeEach(function () {
				_this.clock = _testsFrameworks.sinon.useFakeTimers();
			});

			afterEach(function () {
				_this.clock.restore();
			});

			it('creates a directive per output', function () {
				var Foo = (function () {
					function Foo() {
						_classCallCheck(this, _Foo);
					}

					var _Foo = Foo;
					Foo = (0, _component.Component)({ selector: 'foo', template: 'x', outputs: ['change', 'otherChange'] })(Foo) || Foo;
					return Foo;
				})();

				var root = (0, _testsInternalUtils.quickRootTestComponent)({
					directives: [Foo]
				});

				root.debugElement.getLocal('(change)Directive')[0].restrict.should.eql('A');
				root.debugElement.getLocal('(otherChange)Directive')[0].restrict.should.eql('A');

				root.debugElement.getLocal('(change)Directive')[0].name.should.eql('(change)');
				root.debugElement.getLocal('(otherChange)Directive')[0].name.should.eql('(otherChange)');
			});

			it('creates a directive triggered by dom event', function () {
				var Foo = (function () {
					function Foo() {
						_classCallCheck(this, _Foo2);
					}

					var _Foo2 = Foo;
					Foo = (0, _component.Component)({ selector: 'foo', template: 'x', outputs: ['output'] })(Foo) || Foo;
					return Foo;
				})();

				var root = (0, _testsInternalUtils.quickRootTestComponent)({
					directives: [Foo],
					template: '<foo ng-init="test.bar=false" (output)="test.bar=true"></foo>'
				});

				root.debugElement.componentInstance.bar.should.be['false'];

				root.debugElement.componentViewChildren[0].nativeElement.dispatchEvent(new CustomEvent('output'));
				_this.clock.tick();

				root.debugElement.componentInstance.bar.should.be['true'];
			});

			it('creates a dasherized directive triggered by dom event', function () {
				var Foo = (function () {
					function Foo() {
						_classCallCheck(this, _Foo3);
					}

					var _Foo3 = Foo;
					Foo = (0, _component.Component)({ selector: 'foo', template: 'x', outputs: ['outputChange'] })(Foo) || Foo;
					return Foo;
				})();

				var root = (0, _testsInternalUtils.quickRootTestComponent)({
					directives: [Foo],
					template: '<foo ng-init="test.bar=false" (output-change)="test.bar=true"></foo>'
				});

				root.debugElement.componentInstance.bar.should.be['false'];

				root.debugElement.componentViewChildren[0].nativeElement.dispatchEvent(new CustomEvent('outputChange'));
				_this.clock.tick();

				root.debugElement.componentInstance.bar.should.be['true'];
			});

			it('creates a directive triggered by event emitter', function () {
				var Foo = (function () {
					function Foo() {
						_classCallCheck(this, _Foo4);

						this.output = new _utilEventEmitter.EventEmitter();
					}

					var _Foo4 = Foo;
					Foo = (0, _component.Component)({ selector: 'foo', template: 'x', outputs: ['output'] })(Foo) || Foo;
					return Foo;
				})();

				var root = (0, _testsInternalUtils.quickRootTestComponent)({
					directives: [Foo],
					template: '<foo ng-init="test.bar=false" (output)="test.bar=true"></foo>'
				});

				root.debugElement.componentInstance.bar.should.be['false'];

				root.debugElement.componentViewChildren[0].componentInstance.output.next();
				_this.clock.tick();

				root.debugElement.componentInstance.bar.should.be['true'];
			});

			it('creates a dasherized directive triggered by event emitter', function () {
				var Foo = (function () {
					function Foo() {
						_classCallCheck(this, _Foo5);

						this.outputChange = new _utilEventEmitter.EventEmitter();
					}

					var _Foo5 = Foo;
					Foo = (0, _component.Component)({ selector: 'foo', template: 'x', outputs: ['outputChange'] })(Foo) || Foo;
					return Foo;
				})();

				var root = (0, _testsInternalUtils.quickRootTestComponent)({
					directives: [Foo],
					template: '<foo ng-init="test.bar=false" (output-change)="test.bar=true"></foo>'
				});

				root.debugElement.componentInstance.bar.should.be['false'];

				root.debugElement.componentViewChildren[0].componentInstance.outputChange.next();
				_this.clock.tick();

				root.debugElement.componentInstance.bar.should.be['true'];
			});

			it('creates a directive triggered by local named event emitter', function () {
				var Foo = (function () {
					function Foo() {
						_classCallCheck(this, _Foo6);

						this.o = new _utilEventEmitter.EventEmitter();
					}

					var _Foo6 = Foo;
					Foo = (0, _component.Component)({ selector: 'foo', template: 'x', outputs: ['o:output'] })(Foo) || Foo;
					return Foo;
				})();

				var root = (0, _testsInternalUtils.quickRootTestComponent)({
					directives: [Foo],
					template: '<foo ng-init="test.bar=false" (output)="test.bar=true"></foo>'
				});

				root.debugElement.componentInstance.bar.should.be['false'];

				root.debugElement.componentViewChildren[0].componentInstance.o.next();
				_this.clock.tick();

				root.debugElement.componentInstance.bar.should.be['true'];
			});

			it('bubbles events if they are dispatched  with bubbles set to true', function () {
				var Bar = (function () {
					function Bar() {
						_classCallCheck(this, _Bar);
					}

					var _Bar = Bar;
					Bar = (0, _component.Component)({ selector: 'bar', template: 'x', outputs: ['barChange'] })(Bar) || Bar;
					return Bar;
				})();

				var Foo = (function () {
					function Foo() {
						_classCallCheck(this, _Foo7);
					}

					var _Foo7 = Foo;
					Foo = (0, _component.Component)({
						selector: 'foo',
						directives: [Bar],
						template: '<bar ng-init="foo.bar=false" (bar-change)="foo.bar=true"></bar>'
					})(Foo) || Foo;
					return Foo;
				})();

				var root = (0, _testsInternalUtils.quickRootTestComponent)({
					directives: [Foo],
					template: '<foo ng-init="test.bar=false" (bar-change)="test.bar=true"></foo>'
				});

				var rootEl = root.debugElement;
				var rootComponent = rootEl.componentInstance;
				var fooEl = rootEl.componentViewChildren[0];
				var fooComponent = fooEl.componentInstance;
				var barEl = fooEl.componentViewChildren[0];
				var barComponent = barEl.componentInstance;

				rootComponent.bar.should.be['false'];
				fooComponent.bar.should.be['false'];

				barEl.nativeElement.dispatchEvent(new CustomEvent('barChange', { bubbles: true }));

				rootComponent.bar.should.be['true'];
				fooComponent.bar.should.be['true'];
			});

			it('does not bubble events triggered by event emitter outputs', function () {
				var Bar = (function () {
					function Bar() {
						_classCallCheck(this, _Bar2);

						this.barChange = new _utilEventEmitter.EventEmitter();
					}

					var _Bar2 = Bar;
					Bar = (0, _component.Component)({ selector: 'bar', template: 'x', outputs: ['barChange'] })(Bar) || Bar;
					return Bar;
				})();

				var Foo = (function () {
					function Foo() {
						_classCallCheck(this, _Foo8);
					}

					var _Foo8 = Foo;
					Foo = (0, _component.Component)({
						selector: 'foo',
						directives: [Bar],
						template: '<bar ng-init="foo.bar=false" (bar-change)="foo.bar=true"></bar>'
					})(Foo) || Foo;
					return Foo;
				})();

				var root = (0, _testsInternalUtils.quickRootTestComponent)({
					directives: [Foo],
					template: '<foo ng-init="test.bar=false" (bar-change)="test.bar=true"></foo>'
				});

				var rootEl = root.debugElement;
				var rootComponent = rootEl.componentInstance;
				var fooEl = rootEl.componentViewChildren[0];
				var fooComponent = fooEl.componentInstance;
				var barEl = fooEl.componentViewChildren[0];
				var barComponent = barEl.componentInstance;

				rootComponent.bar.should.be['false'];
				fooComponent.bar.should.be['false'];

				barComponent.barChange.next();
				_this.clock.tick();

				rootComponent.bar.should.be['false'];
				fooComponent.bar.should.be['true'];
			});
		});
	});
});