// # Directive Decorator
//
// ## Usage
//
// ## Setup
// `parseSelector` takes some simple CSS selector and returns a camelCased version
// of the selector as well as the type of selector it was (element, attribute, or
// CSS class).
'use strict';

Object.defineProperty(exports, '__esModule', {
	value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i]; return arr2; } else { return Array.from(arr); } }

var _utilParseSelector = require('../../util/parse-selector');

var _utilParseSelector2 = _interopRequireDefault(_utilParseSelector);

// `providerWriter` sets up provider information, `componentWriter` writes the DDO,
// and `appWriter` sets up app traversal/bootstrapping information.

var _writers = require('../../writers');

// Takes the information from `config.providers` and turns it into the actual metadata
// needed during app traversal

var _Providers = require('../Providers');

// Provider parser will need to be registered with Module

var _module2 = require('../../module');

var _module3 = _interopRequireDefault(_module2);

var _utilDirectiveController = require('../../util/directive-controller');

var _utilDirectiveController2 = _interopRequireDefault(_utilDirectiveController);

var _utilInputsBuilder = require('../../util/inputs-builder');

var _extend = require('extend');

var _extend2 = _interopRequireDefault(_extend);

// The type for right now is `directive`. In angular-decorators there was very little
// difference between `@Component` and `@Directive` so they shared a common provider
// parser defined in `../../util/decorate-directive.js`
var TYPE = 'directive';

// ## Decorator Definition
var Directive = function Directive(componentConfig) {
	return function (t) {
		// The only required config is a selector. If one wasn't passed, throw immediately
		if (!componentConfig.selector) {
			throw new Error('Directive selector must be provided');
		}

		var DEFAULT_CONFIG = {
			providers: []
		};

		var config = (0, _extend2['default'])({}, DEFAULT_CONFIG, componentConfig || {});

		// Grab the provider name and selector type by parsing the selector

		var _parseSelector = (0, _utilParseSelector2['default'])(config.selector);

		var name = _parseSelector.name;
		var restrict = _parseSelector.type;

		// If the selector type was not an element, throw an error. Components can only
		// be elements in Angular 2, so we want to enforce that strictly here.
		if (restrict !== 'A') {
			throw new Error('@Directive selectors can only be attributes');
		}

		if (config.providers !== undefined && !Array.isArray(config.providers)) {
			throw new TypeError('Directive providers must be an array');
		}

		// Setup provider information using the parsed selector
		_writers.providerWriter.set('name', name, t);
		_writers.providerWriter.set('type', TYPE, t);

		// Grab the providers from the config object, parse them, and write the metadata
		// to the target.
		_Providers.Providers.apply(undefined, _toConsumableArray(config.providers))(t);

		// Restrict type must be 'element'
		_writers.componentWriter.set('restrict', restrict, t);
	};
};

exports.Directive = Directive;
// ## Component Provider Parser
_module3['default'].addProvider(TYPE, function (target, name, injects, ngModule) {
	// First create an empty object to contain the directive definition object
	var ddo = {};

	_writers.componentWriter.forEach(function (val, key) {
		// Loop through the key/val pairs of metadata and assign it to the DDO
		ddo[key] = val;
	}, target);

	// Get the input bindings ahead of time
	if (ddo.controllerAs) {
		ddo.bindToController = (0, _utilInputsBuilder.inputsMap)(ddo.inputs);
	}

	// Finally add the directive to the raw module
	ngModule.directive(name, ['$injector', function ($injector) {
		// Component controllers must be created from a factory. Checkout out
		// util/directive-controller.js for more information about what's going on here
		ddo.link = function ($scope, $element, $attrs, $requires, $transclude) {
			return (0, _utilDirectiveController2['default'])(this, injects, target, ddo, $injector, {
				$scope: $scope,
				$element: $element,
				$attrs: $attrs,
				$transclude: $transclude,
				$requires: $requires
			});
		};

		return ddo;
	}]);
});