// # Pipe Decorator
// While not even close to a complete polyfill of Angular 2 pipes, for pure
// filter functions you can begin writing them using pipe-like syntax.
//
// ## Usage
// ```js
// @Pipe('toUpperCase')
// class ToUpperCase{
// 	supports(input){
// 		return typeof input === 'string';
// 	}
//
// 	transform(input){
// 		return input.toUpperCase();
// 	}
// }
// ```
// And in your templates:
// ```html
// {{ vm.name | toUpperCase }}
// ```
// ## Setup
// This is a provider-type decorator, so we'll need to register it with Module
'use strict';

Object.defineProperty(exports, '__esModule', {
	value: true
});
var _bind = Function.prototype.bind;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i]; return arr2; } else { return Array.from(arr); } }

var _module2 = require('../../module');

var _module3 = _interopRequireDefault(_module2);

// The only configurable information it needs is an optional name, so we'll
// generate the decorator with our decorator factory.

var _utilDecoratorFactory = require('../../util/decorator-factory');

var _utilDecoratorFactory2 = _interopRequireDefault(_utilDecoratorFactory);

// ## Decorator Definition
// Provider type for for this decorator is `pipe`, though it is most analogous
// to an Angular 1 filter.
var TYPE = 'pipe';

// The decorator itself. Note that while the name is technically optional,
// with pipes you will almost _always_ wants to provide a name to use in your
// templates that is different from the class name. This keeps your code uglify-proof.
var Pipe = (0, _utilDecoratorFactory2['default'])(TYPE);

exports.Pipe = Pipe;
// ## Provider Parser
_module3['default'].addProvider(TYPE, function (provider, name, injects, ngModule) {
	// This provider recipe uses Angular 1 filters
	ngModule.filter(name, [].concat(_toConsumableArray(injects), [function () {
		for (var _len = arguments.length, dependencies = Array(_len), _key = 0; _key < _len; _key++) {
			dependencies[_key] = arguments[_key];
		}

		// First, create an instance of the provider by passing in injected dependencies
		var pipe = new (_bind.apply(provider, [null].concat(dependencies)))();

		// All pipes must implement a `transform` method. These __must__ be pure
		// functions (for some input, it must always return the same output)
		if (!pipe.transform) {
			throw new Error('Filters must implement a transform method');
		}

		// This is the Angular 1 filter itself
		return function (input) {
			for (var _len2 = arguments.length, params = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
				params[_key2 - 1] = arguments[_key2];
			}

			// Pass the input to the pipe to see if it conforms to the pipe's type
			// spec
			if (pipe.supports && !pipe.supports(input)) {
				throw new Error('Filter ' + name + ' does not support ' + input);
			}

			// Pass all inputs and parameters to the filter returning the output
			return pipe.transform.apply(pipe, [input].concat(params));
		};
	}]));
});