'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i]; return arr2; } else { return Array.from(arr); } }

var _classesProvider = require('../classes/provider');

/**
 * Stores references to all bindings. Is cleared by TestComponentBuilder after a create call.
 * @type {Array}
 * @private
 */
var _providers = [];

/**
 * A sugar function for use in a beforeEach block. It's passed the bind method for
 * creating bindings. Can use in one of two ways:
 *
 * beforeEach(bindings(bind => {}));
 * or
 * beforeEach(() => {
 *   bindings(bind => {});
 * })
 * @param provideFn(provide):[Provider]
 * @returns {workFn}
 */
var providers = function providers(provideFn) {
  return isSpecRunning() ? workFn() : workFn;
  function workFn() {
    var _providers2;

    (_providers2 = _providers).push.apply(_providers2, _toConsumableArray(provideFn(_classesProvider.provide)));
  }
};

exports.providers = providers;
providers.all = function () {
  return _providers;
};

providers.clear = function () {
  _providers = [];
};

// helpers for mocha and jasmine beforeEach
var currentSpec = null;
function isSpecRunning() {
  return !!currentSpec;
}
if (window.jasmine || window.mocha) {
  (window.beforeEach || window.setup)(function () {
    currentSpec = this;
  });
  (window.afterEach || window.teardown)(function () {
    currentSpec = null;
  });
}