'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var _bundle = require('../bundle');

var _bundle2 = _interopRequireDefault(_bundle);

var _providers = require('./providers');

var _writers = require('../writers');

var _decoratorsComponentView = require('../decorators/component/view');

var _extend = require('extend');

var _extend2 = _interopRequireDefault(_extend);

/**
 * TestComponentBuilder
 *
 * The preferred way to test components
 */

var TestComponentBuilder = (function () {
  function TestComponentBuilder() {
    _classCallCheck(this, TestComponentBuilder);

    this._providersOverrides = {};
  }

  /**
   * RootTestComponent is what is returned from a TestComponentBuilder.create call.
   * It gives access to the root test debug element, which in turn gives access to
   * component instance, children and angular element methods. Also has a detectChanges
   * method that triggers a digest.
   */

  _createClass(TestComponentBuilder, [{
    key: 'create',

    /**
     * Takes a root component, typically a test component whose template houses another component
     * under test. Returns a RootTestComponent that contains a debugElement reference
     * to the test component (which you can use to drill down to the component under test) as well
     * as a detectChanges method which aliases to a scope digest call.
     *
     * @param rootComponent
     * @returns {RootTestComponent}
     */
    value: function create(rootComponent) {
      var decoratedModule = (0, _bundle2['default'])('test.module', rootComponent);
      angular.mock.module(decoratedModule.name);
      angular.mock.module(function ($provide) {
        return _providers.providers.all().forEach(function (_ref) {
          var token = _ref.token;
          var useValue = _ref.useValue;
          return $provide.value(token, useValue);
        });
      });

      var rootTC = compileComponent(rootComponent);
      _providers.providers.clear();
      return rootTC;
    }
  }, {
    key: 'overrideTemplate',
    value: function overrideTemplate(component, template) {
      _writers.componentWriter.set('template', template, component);
      return this;
    }
  }, {
    key: 'overrideProviders',
    value: function overrideProviders(component, providers) {
      _writers.appWriter.set('providers', providers, component);
      return this;
    }
  }, {
    key: 'overrideView',
    value: function overrideView(component, config) {
      (0, _decoratorsComponentView.View)(config)(component);
      return this;
    }
  }, {
    key: 'overrideDirective',
    value: function overrideDirective() {
      throw new Error('Method not supported in ng-forward.');
    }
  }, {
    key: 'overrideViewBindings',
    value: function overrideViewBindings() {
      throw new Error('Method not supported in ng-forward.');
    }
  }]);

  return TestComponentBuilder;
})();

exports.TestComponentBuilder = TestComponentBuilder;

var RootTestComponent = (function () {
  function RootTestComponent(_ref2) {
    var debugElement = _ref2.debugElement;
    var rootTestScope = _ref2.rootTestScope;
    var $injector = _ref2.$injector;

    _classCallCheck(this, RootTestComponent);

    this.debugElement = debugElement;
    this.debugElement.data('$injector', $injector);
    this._rootTestScope = rootTestScope;
  }

  /**
   * A function for compiling a decorated component into a RootTestComponent
   *
   * @param ComponentClass
   * @returns {RootTestComponent}
   */

  /**
   * Triggers a root test scope digest.
   */

  _createClass(RootTestComponent, [{
    key: 'detectChanges',
    value: function detectChanges() {
      this._rootTestScope.$digest();
    }
  }]);

  return RootTestComponent;
})();

exports.RootTestComponent = RootTestComponent;
var compileComponent = function compileComponent(ComponentClass) {

  var selector = _writers.appWriter.get('selector', ComponentClass);
  var rootTestScope = undefined,
      debugElement = undefined,
      componentInstance = undefined,
      $injector = undefined;

  inject(function ($compile, $rootScope, _$injector_) {
    var controllerAs = _writers.componentWriter.get('controllerAs', ComponentClass);
    componentInstance = new ComponentClass();
    rootTestScope = $rootScope.$new();
    debugElement = angular.element('<' + selector + '></' + selector + '>');
    debugElement = $compile(debugElement)(rootTestScope);
    rootTestScope.$digest();
    $injector = _$injector_;
  });

  return new RootTestComponent({ debugElement: debugElement, rootTestScope: rootTestScope, $injector: $injector });
};

exports.compileComponent = compileComponent;
/**
 * A function for compiling an html template against a data object. This is
 * for testing directives in regular angular 1. No dependencies on the
 * ng-forward.
 *
 * Recommended to use TestComponentBuilder instead if you are using ng-forward.
 *
 * @param html
 * @param initialScope
 * @param selector
 * @returns {{parentScope: *, element: *, controller: *, isolateScope: *}}
 */
var compileHtmlAndScope = function compileHtmlAndScope(_ref3) {
  var html = _ref3.html;
  var initialScope = _ref3.initialScope;
  var selector = _ref3.selector;

  var parentScope = undefined,
      element = undefined,
      controller = undefined,
      isolateScope = undefined;

  inject(function ($compile, $rootScope) {
    parentScope = $rootScope.$new();
    (0, _extend2['default'])(parentScope, initialScope);
    element = angular.element(html);
    element = $compile(element)(parentScope);
    parentScope.$digest();
    isolateScope = element.isolateScope();
    controller = element.controller('' + selector);
  });

  return { parentScope: parentScope, element: element, controller: controller, isolateScope: isolateScope };
};
exports.compileHtmlAndScope = compileHtmlAndScope;