/* global it, describe */
'use strict';

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

require('./frameworks');

var _decoratorsProvidersComponent = require('../decorators/providers/component');

var _decoratorsComponentView = require('../decorators/component/view');

var _decoratorsInject = require('../decorators/inject');

var _decoratorsProvidersInjectable = require('../decorators/providers/injectable');

var _angular = require('./angular');

var _index = require('./index');

var _testComponentBuilder = require('./test-component-builder');

var _extend = require('extend');

var _extend2 = _interopRequireDefault(_extend);

describe('Test Utils', function () {

  var tcb = undefined;
  var angular = undefined;
  var SomeService = undefined;
  var SomeOtherService = undefined;
  var SomeComponent = undefined;
  var TestComponent = undefined;

  beforeEach(function () {
    tcb = new _index.TestComponentBuilder();
    angular = _angular.ng.useReal();

    SomeService = (function () {
      function SomeService() {
        _classCallCheck(this, _SomeService);
      }

      _createClass(SomeService, [{
        key: 'getData',
        value: function getData() {
          return 'real success';
        }
      }]);

      var _SomeService = SomeService;
      SomeService = (0, _decoratorsProvidersInjectable.Injectable)()(SomeService) || SomeService;
      return SomeService;
    })();

    SomeOtherService = (function () {
      function SomeOtherService() {
        _classCallCheck(this, _SomeOtherService);
      }

      _createClass(SomeOtherService, [{
        key: 'getData',
        value: function getData() {
          return 'real other';
        }
      }]);

      var _SomeOtherService = SomeOtherService;
      SomeOtherService = (0, _decoratorsProvidersInjectable.Injectable)()(SomeOtherService) || SomeOtherService;
      return SomeOtherService;
    })();

    SomeComponent = (function () {
      function SomeComponent(SomeService, SomeOtherService, $http, $timeout) {
        var _this = this;

        _classCallCheck(this, _SomeComponent);

        (0, _extend2['default'])(this, { SomeService: SomeService, SomeOtherService: SomeOtherService, $http: $http, $timeout: $timeout });
        this.local = 'a';
        $http.get('/api');
        $timeout(function () {
          return _this.local = 'c';
        }, 1000);
      }

      _createClass(SomeComponent, [{
        key: 'quux',
        value: function quux() {
          return this.SomeService.getData() + ' ' + this.SomeOtherService.getData();
        }
      }]);

      var _SomeComponent = SomeComponent;
      SomeComponent = (0, _decoratorsInject.Inject)(SomeService, SomeOtherService, '$http', '$timeout')(SomeComponent) || SomeComponent;
      SomeComponent = (0, _decoratorsComponentView.View)({
        template: '{{someComponent.foo}} {{someComponent.baz}} {{someComponent.quux()}} {{someComponent.local}}'
      })(SomeComponent) || SomeComponent;
      SomeComponent = (0, _decoratorsProvidersComponent.Component)({
        selector: 'some-component',
        inputs: ['foo', 'baz:bar'],
        bindings: [SomeService]
      })(SomeComponent) || SomeComponent;
      return SomeComponent;
    })();

    TestComponent = (function () {
      function TestComponent() {
        _classCallCheck(this, _TestComponent);

        this.bar = "World";
      }

      var _TestComponent = TestComponent;
      TestComponent = (0, _decoratorsComponentView.View)({
        template: '<some-component foo="Hello" [bar]="test.bar"></some-component>',
        directives: [SomeComponent]
      })(TestComponent) || TestComponent;
      TestComponent = (0, _decoratorsProvidersComponent.Component)({ selector: 'test' })(TestComponent) || TestComponent;
      return TestComponent;
    })();
  });

  describe('Test Component Builder', function () {
    var mockSomeService = undefined;
    var mockSomeOtherService = undefined;
    var $http = undefined;
    var $timeout = undefined;
    var rootTC = undefined;
    var rootTestEl = undefined;
    var someComponentEl = undefined;

    // test the bindings call composed with the beforeEach fn
    beforeEach((0, _index.providers)(function (provide) {
      mockSomeService = {
        getData: sinon.stub().returns('mock success')
      };

      $http = { get: sinon.stub() };

      return [provide(SomeService, { useValue: mockSomeService }), provide('$http', { useValue: $http })];
    }));

    // testing adding more bindings in an additional beforeEach
    beforeEach(function () {

      // test the bindings call inside the beforeEach fn
      (0, _index.providers)(function (provide) {
        mockSomeOtherService = {
          getData: sinon.stub().returns('mock other')
        };

        return [provide(SomeOtherService, { useValue: mockSomeOtherService })];
      });
    });

    beforeEach(function () {
      rootTC = tcb.create(TestComponent);
      rootTestEl = rootTC.debugElement;
      someComponentEl = rootTC.debugElement.componentViewChildren[0];
    });

    // todo: write a custom inject function for ng-forward
    // currently I'm just using angular.mock.inject
    beforeEach(inject(function (_$timeout_) {
      $timeout = _$timeout_;
    }));

    it('should bootstrap the test module', function () {
      expect(angular.module('test.module')).to.exist;
    });

    it('should return a root test component and decorated jqlite', function () {
      expect(rootTC).to.be.an.instanceOf(_testComponentBuilder.RootTestComponent);

      // debugElement is an angular.element decorated with extra properties, see next lines
      expect(rootTC.debugElement).to.be.an.instanceOf(angular.element);

      // nativeElement is an alias to the [0] index raw dom element
      expect(rootTC.debugElement.nativeElement).to.be.an.instanceOf(HTMLElement);

      // The actual class instance hosted by the element
      expect(rootTC.debugElement.componentInstance).to.be.an.instanceOf(TestComponent);

      // componentViewChildren is an alias to .children()
      expect(rootTC.debugElement.componentViewChildren[0]).to.be.an.instanceOf(angular.element);

      // getLocal is an alias to $injector
      expect(rootTC.debugElement.getLocal('$q')).to.contain.all.keys(['resolve', 'reject', 'defer']);

      // Checking to be sure even nested jqlite elements are decorated
      expect(someComponentEl.nativeElement).to.be.an.instanceOf(HTMLElement);

      expect(someComponentEl.componentInstance).to.be.an.instanceOf(SomeComponent);

      expect(someComponentEl.componentViewChildren).to.be.empty;

      expect(someComponentEl.getLocal('$q')).to.contain.all.keys(['resolve', 'reject', 'defer']);
    });

    it('should allow mock decorated class components and services via bindings() method', function () {
      expect(mockSomeService.getData).to.have.been.called;
      expect(someComponentEl.text()).to.equal("Hello World mock success mock other a");
    });

    it('should allow mock angular 1 services via bindings() method', function () {
      expect($http.get).to.have.been.called;
    });

    it('should allow angular.mock special services (e.g. $timeout.flush)', function () {
      expect(someComponentEl.text()).to.equal("Hello World mock success mock other a");
      $timeout.flush();
      expect(someComponentEl.text()).to.equal("Hello World mock success mock other c");
    });

    it('should detect changes on root test component instance ', function () {
      expect(someComponentEl.text()).to.equal("Hello World mock success mock other a");

      rootTestEl.componentInstance.bar = "Angular 2";
      rootTC.detectChanges();

      expect(someComponentEl.text()).to.equal("Hello Angular 2 mock success mock other a");
    });

    it('should detect changes on component instance under test', function () {
      expect(someComponentEl.text()).to.equal("Hello World mock success mock other a");

      someComponentEl.componentInstance.local = "b";
      rootTC.detectChanges();

      expect(someComponentEl.text()).to.equal("Hello World mock success mock other b");
    });
  });
});