/* global Object */
// # Directive Controller Factory
// While we want to use the component controller, we need a way to add our own
// properties to the controller instance before the constructor is called. We also
// want to do this in the context of Angular's DI so that we can access the $element
// for events and $filter for properties.
//
// ## Setup
// We'll need a1atscript's inputsBuilder for generating the property definitions
'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = createDirectiveController;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i]; return arr2; } else { return Array.from(arr); } }

var _inputsBuilder = require('./inputs-builder');

// Also need the outputsBuilder for creating event emitters

var _outputsBuilder = require('./outputs-builder');

var _outputsBuilder2 = _interopRequireDefault(_outputsBuilder);

// Finally extend for extending the instance of the controller

var _extend = require('extend');

var _extend2 = _interopRequireDefault(_extend);

// ## Factory
// Needs the injection array, the controller class, and the directive definition
// object in order to generate the controller

function createDirectiveController(caller, injects, controller, ddo, $injector, locals) {
  // Create an instance of the controller without calling its constructor
  var instance = Object.create(controller.prototype);
  // Use a1atscript's inputsBuilder to add the getters/setters then sugar
  // over `=` and `@` bindings
  for (var key in ddo.inputMap) {
    (0, _inputsBuilder.inputsBuilder)(instance, key, ddo.inputMap[key]);
  }
  // Remember, angular has already set those bindings on the `caller`
  // argument. Now we need to extend them onto our `instance`. It is important
  // to extend after defining the properties. That way we fire the setters.
  (0, _extend2['default'])(instance, caller);

  // Finally, invoke the constructor using the injection array and the captured
  // locals
  $injector.invoke([].concat(_toConsumableArray(injects), [controller]), instance, locals);

  // Outputs work similarly, but they need the raw $element and the $scope for
  // destroying output observables.
  var $element = locals.$element;
  var $scope = locals.$scope;

  (0, _outputsBuilder2['default'])(instance, $element[0], $scope, ddo.outputMap || {});

  // Return the controller instance
  return instance;
}

module.exports = exports['default'];