'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _writers = require('../writers');

// ## Intro
// Import the `@Injectable` decorator. We'll apply it to functions/classes that are
// injected that are missing provider metadata. Convenience!

var _decoratorsProvidersInjectable = require('../decorators/providers/injectable');

var _classesOpaqueToken = require('../classes/opaque-token');

var getInjectableName = function getInjectableName(injectable) {
  // Return it if it is already a string like `'$http'` or `'$state'`
  if (typeof injectable === 'string' || injectable instanceof _classesOpaqueToken.OpaqueToken) {
    return injectable.toString();
  }
  // If the injectable is not a string but has provider information, use
  // the provider name. This is set by the collection of provider decorators
  else if (_writers.providerWriter.has('type', injectable)) {
      return _writers.providerWriter.get('name', injectable);
    }
};

exports.getInjectableName = getInjectableName;
var getInjectableNameWithJitCreation = function getInjectableNameWithJitCreation(injectable) {
  var name = getInjectableName(injectable);

  if (name) {
    return name;
  }

  // If it is a function but is missing provider information, apply the Injectable
  // provider decorator to the function to turn it into a service.
  if (typeof injectable === 'function') {
    (0, _decoratorsProvidersInjectable.Injectable)(injectable);
    return _writers.providerWriter.get('name', injectable);
  }
};
exports.getInjectableNameWithJitCreation = getInjectableNameWithJitCreation;