/* global describe, it */
'use strict';

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

require('../tests/frameworks');

var _groupModulesProviders = require('./group-modules-providers');

var _groupModulesProviders2 = _interopRequireDefault(_groupModulesProviders);

var _writers = require('../writers');

var _classesProvider = require('../classes/provider');

var Test = function Test(t) {
  _writers.providerWriter.set('type', 'test', t);
  _writers.providerWriter.set('name', t.name, t);
};

describe('groupIntoModulesAndProviders Utility', function () {
  it('should separate providers and string-based module names', function () {
    var Example = (function () {
      function Example() {
        _classCallCheck(this, _Example);
      }

      var _Example = Example;
      Example = Test(Example) || Example;
      return Example;
    })();

    var p = new _classesProvider.Provider('foo', { useValue: 'bar' });

    var _groupModulesAndProviders = (0, _groupModulesProviders2['default'])(['ui-router', 'ui-bootstrap', p, Example]);

    var modules = _groupModulesAndProviders.modules;
    var providers = _groupModulesAndProviders.providers;

    modules.should.eql(['ui-router', 'ui-bootstrap']);
    providers.should.eql([p, Example]);
  });

  it('should flatten the array if it is an array of arrays', function () {
    var Example = (function () {
      function Example() {
        _classCallCheck(this, _Example2);
      }

      var _Example2 = Example;
      Example = Test(Example) || Example;
      return Example;
    })();

    var p = new _classesProvider.Provider('foo', { useValue: 'bar' });

    var _groupModulesAndProviders2 = (0, _groupModulesProviders2['default'])([['ui-router'], 'ui-bootstrap', [Example, [p]]]);

    var modules = _groupModulesAndProviders2.modules;
    var providers = _groupModulesAndProviders2.providers;

    modules.should.eql(['ui-router', 'ui-bootstrap']);
    providers.should.eql([Example, p]);
  });
});